package biz

import (
	"context"
	"time"

	"github.com/go-kratos/kratos/v2/log"

	"demo/internal/biz/common"
	"demo/internal/biz/errs"
	"demo/internal/util"
)

type TimeGranularity string

const (
	StatTimeGranularityDaily  TimeGranularity = "STAT_TIME_GRANULARITY_DAILY"  // 按天维度
	StatTimeGranularityHourly TimeGranularity = "STAT_TIME_GRANULARITY_HOURLY" // 按小时维度
)

type ListCampaignRequest struct {
	common.PageRequest
	TimeGranularity TimeGranularity `form:"time_granularity" json:"time_granularity"` // 时间粒度(0:按天维度;1:按小时维度)
	StartDate       string          `form:"start_date" json:"start_date"`             // 开始时间(格式YYYY-MM-DD)
	EndDate         string          `form:"end_date" json:"end_date"`                 // 结束时间(格式YYYY-MM-DD)
	StartDateLong   util.Second     `json:"-"`                                        // 开始时间
	EndDateLong     util.Second     `json:"-"`                                        // 结束时间
}

type ListCampaignResponse struct {
	List     CampaignList     `json:"list"`      // 数据信息
	PageInfo *common.PageInfo `json:"page_info"` // 页码信息
}

type CampaignList []*Campaign

type Campaign struct {
	CampaignId                            string    `json:"campaign_id"`                                 // 广告组ID
	CampaignName                          string    `json:"campaign_name"`                               // 广告主组名称
	AdvertiserId                          string    `json:"advertiser_id"`                               // 广告主id
	StatDatetime                          string    `json:"stat_datetime"`                               // 数据起始时间(yyyy-MM-dd HH:mm:ss)
	Cost                                  float64   `json:"cost"`                                        // 花费
	Show                                  int       `json:"show"`                                        // 展示量
	AvgShowCost                           float64   `json:"avg_show_cost"`                               // 千次展示成本
	Click                                 int       `json:"click"`                                       // 点击量
	AvgClickCost                          float64   `json:"avg_click_cost"`                              // 点击成本
	Ctr                                   float64   `json:"ctr"`                                         // 点击率
	Convert                               int       `json:"convert"`                                     // 转化数
	ConvertCost                           float64   `json:"convert_cost"`                                // 转化成本
	ConvertRate                           float64   `json:"convert_rate"`                                // 转化率
	DeepConvert                           int       `json:"deep_convert"`                                // 深度转化数
	DeepConvertCost                       float64   `json:"deep_convert_cost"`                           // 深度转化成本
	DeepConvertRate                       float64   `json:"deep_convert_rate"`                           // 深度转化率
	AttributionConvert                    int       `json:"attribution_convert"`                         // 转化数（计费时间）
	AttributionConvertCost                float64   `json:"attribution_convert_cost"`                    // 转化成本（计费时间）
	AttributionDeepConvert                int       `json:"attribution_deep_convert"`                    // 深度转化数（计费时间）
	AttributionDeepConvertCost            float64   `json:"attribution_deep_convert_cost"`               // 深度转化成本（计费时间）
	DownloadStart                         int       `json:"download_start"`                              // 安卓下载开始数
	DownloadStartCost                     float64   `json:"download_start_cost"`                         // 安卓下载开始成本
	DownloadStartRate                     float64   `json:"download_start_rate"`                         // 安卓下载开始率
	DownloadFinish                        int       `json:"download_finish"`                             // 安卓下载完成数
	DownloadFinishCost                    float64   `json:"download_finish_cost"`                        // 安卓下载完成成本
	DownloadFinishRate                    float64   `json:"download_finish_rate"`                        // 安卓下载完成率
	ClickInstall                          int       `json:"click_install"`                               // 点击安装数
	InstallFinish                         int       `json:"install_finish"`                              // 安卓安装完成数
	InstallFinishCost                     float64   `json:"install_finish_cost"`                         // 安卓安装完成成本
	InstallFinishRate                     float64   `json:"install_finish_rate"`                         // 安卓安装完成率
	Active                                int       `json:"active"`                                      // 激活数
	ActiveCost                            float64   `json:"active_cost"`                                 // 激活成本
	ActiveRate                            float64   `json:"active_rate"`                                 // 激活率
	Register                              int       `json:"register"`                                    // 注册数
	ActiveRegisterCost                    float64   `json:"active_register_cost"`                        // 注册成本
	ActiveRegisterRate                    float64   `json:"active_register_rate"`                        // 注册率
	NextDayOpen                           int       `json:"next_day_open"`                               // 次留数
	NextDayOpenCost                       float64   `json:"next_day_open_cost"`                          // 次留成本
	NextDayOpenRate                       float64   `json:"next_day_open_rate"`                          // 次留率
	AttributionNextDayOpenCnt             int       `json:"attribution_next_day_open_cnt"`               // 次留回传数
	AttributionNextDayOpenCost            float64   `json:"attribution_next_day_open_cost"`              // 次留成本
	AttributionNextDayOpenRate            float64   `json:"attribution_next_day_open_rate"`              // 次留率
	GameAddiction                         int       `json:"game_addiction"`                              // 关键行为数
	GameAddictionCost                     float64   `json:"game_addiction_cost"`                         // 关键行为成本
	GameAddictionRate                     float64   `json:"game_addiction_rate"`                         // 关键行为率
	PayCount                              int       `json:"pay_count"`                                   // 首次付费次数
	ActivePayCost                         float64   `json:"active_pay_cost"`                             // 首次付费成本
	ActivePayRate                         float64   `json:"active_pay_rate"`                             // 首次付费率
	InAppUv                               int       `json:"in_app_uv"`                                   // APP内访问
	InAppDetailUv                         int       `json:"in_app_detail_uv"`                            // APP内访问详情页
	InAppCart                             int       `json:"in_app_cart"`                                 // APP内加入购物车
	InAppPay                              int       `json:"in_app_pay"`                                  // APP内付费
	InAppOrder                            int       `json:"in_app_order"`                                // APP内下单
	AttributionGamePay7DCount             int       `json:"attribution_game_pay_7d_count"`               // 7日付费次数（激活时间）
	GamePayCount                          int       `json:"game_pay_count"`                              // 付费次数
	TotalPlay                             int       `json:"total_play"`                                  // 视频数据-播放数
	ValidPlay                             int       `json:"valid_play"`                                  // 视频数据-有效播放数
	ValidPlayCost                         float64   `json:"valid_play_cost"`                             // 有效播放成本
	ValidPlayRate                         float64   `json:"valid_play_rate"`                             // 有效播放率
	Play25FeedBreak                       int       `json:"play_25_feed_break"`                          // 视频数据-25%进度播放数
	Play50FeedBreak                       int       `json:"play_50_feed_break"`                          // 视频数据-50%进度播放数
	Play75FeedBreak                       int       `json:"play_75_feed_break"`                          // 视频数据-75%进度播放数
	Play100FeedBreak                      int       `json:"play_100_feed_break"`                         // 视频数据-99%进度播放数
	AveragePlayTimePerPlay                float64   `json:"average_play_time_per_play"`                  // 视频数据-平均单次播放时长，单位：秒
	PlayOverRate                          float64   `json:"play_over_rate"`                              // 播完率
	PlayDurationSum                       int       `json:"play_duration_sum"`                           // 视频数据-播放时长，单位ms
	AttributionActivePayIntraOneDayCount  int       `json:"attribution_active_pay_intra_one_day_count"`  // 游戏行业-激活后24h首次付费数
	AttributionActivePayIntraOneDayAmount float64   `json:"attribution_active_pay_intra_one_day_amount"` // 游戏行业-激活后24h付费金额
	Timestamp                             time.Time `json:"timestamp"`                                   // 创建时间
}

// CampaignRepo is a Report camRepo.
type CampaignRepo interface {
	List(ctx context.Context, customerId int64, req *ListCampaignRequest) (int64, CampaignList, error)
}

// ReportUsecase is a Report usecase.
type ReportUsecase struct {
	camRepo CampaignRepo
	log     *log.Helper
}

// NewReportUsecase new a Report usecase.
func NewReportUsecase(repo CampaignRepo, logger log.Logger) *ReportUsecase {
	return &ReportUsecase{camRepo: repo, log: log.NewHelper(logger)}
}

// ListCampaignData list campaign data.
func (p *ReportUsecase) ListCampaignData(ctx context.Context, customerId int64, req *ListCampaignRequest) (*ListCampaignResponse, error) {
	p.log.WithContext(ctx).Infof("req: %v", req)
	if req.StartDate != "" {
		parse, err := time.Parse("2006-01-02", req.StartDate)
		if err != nil {
			return nil, errs.ErrorArgs.WithMessage("开始时间格式错误")
		}
		req.StartDateLong = util.NewSecond(parse)
	}
	if req.EndDate != "" {
		parse, err := time.Parse("2006-01-02", req.EndDate)
		if err != nil {
			return nil, errs.ErrorArgs.WithMessage("结束时间格式错误")
		}
		req.EndDateLong = util.NewSecond(parse)
	}

	total, list, err := p.camRepo.List(ctx, customerId, req)
	if err != nil {
		return nil, err
	}
	return &ListCampaignResponse{
		List:     list,
		PageInfo: req.PageInfo(total),
	}, nil
}
